(* scanner.mll *)
(* @Author:Liming Zhang @version 0.0: keywords: TRUE, FALSE, MAIN @version 0.1: add NOR*)

{ open Parser }

(* letter, digit *)
let letter = ['a'-'z' 'A'-'Z']
let digit = ['0'-'9']
let quotes=['"']
let character = ['\b' '\t' '\n' '\r' '\\' ' ' '!' '@' '#' '$' '%' '^' '&' '*' '(' ')' '-' '_' '+' '=' '{' '[' '}' ']' '|' ';' ':' '<' '>' '.' ',' '?' '/' '~' '`']
            | letter | digit

rule token = parse 
	[' ' '\t' '\r' '\n'] 	{ token lexbuf }	 		(*Whitespace*)
	| "/*"     { comment lexbuf }           (* Comments *)
  | "//"     { line_comment lexbuf }
	(* keywords *)
	|"if"        { IF }
  |"else"      { ELSE }
  |"for"       { FOR }
  |"while"     { WHILE }
  |"return"    { RETURN }
  |"int"       { INT }
  |"float"     { FLOAT }
  |"string"    { STRING }
  |"new"       { NEW }
  |"switch"    { SWITCH }
  |"case"      { CASE }
  |"res"       { RES }
  |"bit"       { BIT }
  |"function"  { FUNCTION }
  |"default"   { DEFAULT  }
  |"value"     { VALUE }
  |"void"      { VOID }
  |"boolean"   { BOOLEAN }
  (* Operators *)
  |'+'         { PLUS }
  |'-'         { MINUS }
  |'*'         { MULTIPLY }
  |'/'         { DIVIDE }
  |'%'         { MODULUS }
  |'~'         { RECIPROCAL }
  |'<'         { LT }
  |'>'         { GT }
  |"<="        { LE }
  |">="        { GE }
  |"!="        { NE }
  |"=="        { EQ }
  |'!'         { NOT }
  |'^'         { XOR }
  |"!^"        { XNOR }
  |'&'         { AND }
  |"!&"        { NAND }
  |'|'         { OR }
  |"!|"        { NOR }
  |'='         { ASSIGN }
  |'#'         { SERIES }
  |'$'         { PARALLEL }
	(*Punctuation*)
	| '(' { LPAREN }	
  | ')' { RPAREN }				
	| '{' { LBRACE }	
  | '}' { RBRACE }
	| '[' { LBRACKET }
	| ']' { RBRACKET }
	| ';' { SEMICOLON } 
  | ',' { COMMA }
	| ':' { COLON }
	| '.' { DOT }

  | digit+  as integer    { INTLiteral(int_of_string integer) }
  | digit+ '.' digit* 
  | '.' digit+ ('e' ['+''-']? digit+)?
  | digit+ ('.' digit+)? 'e' ['+' '-']? digit+   as float  { FLOATLiteral(float_of_string float) }
	| quotes ['0' '1']+ quotes 'b' as var { BITLiteral (String.sub var 0 (String.length var - 1) )  }
  | letter (letter | digit | '_')* as identifier { ID(identifier) }
| quotes (letter|digit|character)* quotes as sentence {Sentence(sentence)}
 	| eof { EOF }
  | _ as err_char { raise (Failure("illegal character " ^ Char.escaped err_char)) }
	

(* comment *)
and line_comment = parse
    '\n'     { token lexbuf }
    | _      { line_comment lexbuf }

and comment = parse
    "*/"     { token lexbuf }
    | _      { comment lexbuf }